@@ -21,14 +21,17 @@
 /**
  * SECTION:element-cairotimeoverlay
  *
+ * <refsect2>
+ * <para>
  * cairotimeoverlay renders the buffer timestamp for each frame on top of
  * the frame.
- *
- * <refsect2>
+ * </para>
  * <title>Example launch line</title>
- * |[
- * gst-launch videotestsrc ! cairotimeoverlay ! autovideosink
- * ]|
+ * <para>
+ * <programlisting>
+ * gst-launch -v -m videotestsrc ! cairotimeoverlay ! autovideosink
+ * </programlisting>
+ * </para>
  * </refsect2>
  */
 
@@ -45,10 +48,6 @@
 
 #include <gst/video/video.h>
 
-#ifndef HAVE_RINT
-#define rint(x) ((double) floor((x)+(((x) < 0)? -0.5 : 0.5)))
-#endif
-
 static const GstElementDetails cairo_time_overlay_details =
 GST_ELEMENT_DETAILS ("Time overlay",
     "Filter/Editor/Video",
@@ -92,7 +91,7 @@ gst_cairo_time_overlay_update_font_height (GstCairoTimeOverlay * timeoverlay)
   cairo_set_font_size (font_cairo, 20);
   cairo_font_extents (font_cairo, &font_extents);
   timeoverlay->text_height = font_extents.height;
-  GST_DEBUG_OBJECT (timeoverlay, "font height is %f", font_extents.height);
+  GST_DEBUG_OBJECT (timeoverlay, "font height is %d", font_extents.height);
   cairo_destroy (font_cairo);
   font_cairo = NULL;
 }
@@ -160,7 +159,7 @@ gst_cairo_time_overlay_print_smpte_time (guint64 time)
   int ms;
   double x;
 
-  x = rint (gst_util_guint64_to_gdouble (time + 500000) * 1e-6);
+  x = rint ((time + 500000) * 1e-6);
 
   hours = floor (x / (60 * 60 * 1000));
   x -= hours * 60 * 60 * 1000;
@@ -187,14 +186,14 @@ gst_cairo_time_overlay_transform (GstBaseTransform * trans, GstBuffer * in,
   int i, j;
   unsigned char *image;
   cairo_text_extents_t extents;
-  guint8 *dest, *src;
+  gpointer dest, src;
   cairo_surface_t *font_surface;
   cairo_t *text_cairo;
   GstFlowReturn ret = GST_FLOW_OK;
 
   timeoverlay = GST_CAIRO_TIME_OVERLAY (trans);
 
-  gst_buffer_copy_metadata (out, in, GST_BUFFER_COPY_TIMESTAMPS);
+  gst_buffer_stamp (out, in);
 
   src = GST_BUFFER_DATA (in);
   dest = GST_BUFFER_DATA (out);
