@@ -21,21 +21,25 @@
 /**
  * SECTION:element-dvdec
  *
+ * <refsect2>
+ * <para>
  * dvdec decodes DV video into raw video. The element expects a full DV frame
  * as input, which is 120000 bytes for NTSC and 144000 for PAL video.
- *
- * This element can perform simple frame dropping with the #GstDVDec:drop-factor
+ * </para>
+ * <para>
+ * This element can perform simple frame dropping with the drop-factor
  * property. Setting this property to a value N > 1 will only decode every 
  * Nth frame.
- *
- * <refsect2>
+ * </para>
  * <title>Example launch line</title>
- * |[
+ * <para>
+ * <programlisting>
  * gst-launch filesrc location=test.dv ! dvdemux name=demux ! dvdec ! xvimagesink
- * ]| This pipeline decodes and renders the raw DV stream to a videosink.
- * </refsect2>
- *
+ * </programlisting>
+ * This pipeline decodes and renders the raw DV stream to a videosink.
+ * </para>
  * Last reviewed on 2006-02-28 (0.10.3)
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +47,6 @@
 #endif
 #include <string.h>
 #include <math.h>
-#include <gst/video/video.h>
 
 #include "gstdvdec.h"
 
@@ -55,30 +58,13 @@ GST_ELEMENT_DETAILS ("DV video decoder",
     "Erik Walthinsen <omega@cse.ogi.edu>," "Wim Taymans <wim@fluendo.com>");
 
 /* sizes of one input buffer */
-#define NTSC_HEIGHT 480
 #define NTSC_BUFFER 120000
-#define NTSC_FRAMERATE_NUMERATOR 30000
-#define NTSC_FRAMERATE_DENOMINATOR 1001
-
-#define PAL_HEIGHT 576
 #define PAL_BUFFER 144000
-#define PAL_FRAMERATE_NUMERATOR 25
-#define PAL_FRAMERATE_DENOMINATOR 1
-
-#define PAL_NORMAL_PAR_X        59
-#define PAL_NORMAL_PAR_Y        54
-#define PAL_WIDE_PAR_X          118
-#define PAL_WIDE_PAR_Y          81
-
-#define NTSC_NORMAL_PAR_X       10
-#define NTSC_NORMAL_PAR_Y       11
-#define NTSC_WIDE_PAR_X         40
-#define NTSC_WIDE_PAR_Y         33
 
 #define DV_DEFAULT_QUALITY DV_QUALITY_BEST
 #define DV_DEFAULT_DECODE_NTH 1
 
-GST_DEBUG_CATEGORY_STATIC (dvdec_debug);
+GST_DEBUG_CATEGORY (dvdec_debug);
 #define GST_CAT_DEFAULT dvdec_debug
 
 enum
@@ -155,10 +141,8 @@ gst_dvdec_quality_get_type (void)
 
 GST_BOILERPLATE (GstDVDec, gst_dvdec, GstElement, GST_TYPE_ELEMENT);
 
-static void gst_dvdec_finalize (GObject * object);
 static gboolean gst_dvdec_sink_setcaps (GstPad * pad, GstCaps * caps);
 static GstFlowReturn gst_dvdec_chain (GstPad * pad, GstBuffer * buffer);
-static gboolean gst_dvdec_sink_event (GstPad * pad, GstEvent * event);
 
 static GstStateChangeReturn gst_dvdec_change_state (GstElement * element,
     GstStateChange transition);
@@ -192,7 +176,6 @@ gst_dvdec_class_init (GstDVDecClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = gst_dvdec_finalize;
   gobject_class->set_property = gst_dvdec_set_property;
   gobject_class->get_property = gst_dvdec_get_property;
 
@@ -210,41 +193,39 @@ gst_dvdec_class_init (GstDVDecClass * klass)
           1, G_MAXINT, DV_DEFAULT_DECODE_NTH, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_dvdec_change_state;
+
+  /* table initialization, only do once */
+  dv_init (0, 0);
 }
 
 static void
 gst_dvdec_init (GstDVDec * dvdec, GstDVDecClass * g_class)
 {
-  dvdec->sinkpad = gst_pad_new_from_static_template (&sink_temp, "sink");
+  dvdec->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&sink_temp),
+      "sink");
   gst_pad_set_setcaps_function (dvdec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_dvdec_sink_setcaps));
   gst_pad_set_chain_function (dvdec->sinkpad, gst_dvdec_chain);
-  gst_pad_set_event_function (dvdec->sinkpad, gst_dvdec_sink_event);
   gst_element_add_pad (GST_ELEMENT (dvdec), dvdec->sinkpad);
 
-  dvdec->srcpad = gst_pad_new_from_static_template (&src_temp, "src");
+  dvdec->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&src_temp),
+      "src");
+
   gst_pad_use_fixed_caps (dvdec->srcpad);
+
   gst_element_add_pad (GST_ELEMENT (dvdec), dvdec->srcpad);
 
   dvdec->framerate_numerator = 0;
   dvdec->framerate_denominator = 0;
+  dvdec->height = 0;
   dvdec->wide = FALSE;
   dvdec->drop_factor = 1;
 
   dvdec->clamp_luma = FALSE;
   dvdec->clamp_chroma = FALSE;
   dvdec->quality = DV_DEFAULT_QUALITY;
-  dvdec->segment = gst_segment_new ();
-}
-
-static void
-gst_dvdec_finalize (GObject * object)
-{
-  GstDVDec *dvdec = GST_DVDEC (object);
-
-  gst_segment_free (dvdec->segment);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
@@ -252,6 +233,8 @@ gst_dvdec_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstDVDec *dvdec;
   GstStructure *s;
+  GstCaps *othercaps;
+  gboolean gotpar = FALSE;
   const GValue *par = NULL, *rate = NULL;
 
   dvdec = GST_DVDEC (gst_pad_get_parent (pad));
@@ -259,67 +242,18 @@ gst_dvdec_sink_setcaps (GstPad * pad, GstCaps * caps)
   /* first parse the caps */
   s = gst_caps_get_structure (caps, 0);
 
-  /* we allow framerate and PAR to be overwritten. framerate is mandatory. */
+  if (!gst_structure_get_int (s, "height", &dvdec->height))
+    goto error;
+  if ((par = gst_structure_get_value (s, "pixel-aspect-ratio")))
+    gotpar = TRUE;
   if (!(rate = gst_structure_get_value (s, "framerate")))
-    goto no_framerate;
-  par = gst_structure_get_value (s, "pixel-aspect-ratio");
+    goto error;
 
-  if (par) {
-    dvdec->par_x = gst_value_get_fraction_numerator (par);
-    dvdec->par_y = gst_value_get_fraction_denominator (par);
-    dvdec->need_par = FALSE;
-  } else {
-    dvdec->par_x = 0;
-    dvdec->par_y = 0;
-    dvdec->need_par = TRUE;
-  }
   dvdec->framerate_numerator = gst_value_get_fraction_numerator (rate);
   dvdec->framerate_denominator = gst_value_get_fraction_denominator (rate);
-  dvdec->sink_negotiated = TRUE;
-  dvdec->src_negotiated = FALSE;
-
-  gst_object_unref (dvdec);
-
-  return TRUE;
-
-  /* ERRORS */
-no_framerate:
-  {
-    GST_DEBUG_OBJECT (dvdec, "no framerate specified in caps");
-    gst_object_unref (dvdec);
-    return FALSE;
-  }
-}
-
-static gboolean
-gst_dvdec_src_negotiate (GstDVDec * dvdec)
-{
-  GstCaps *othercaps;
-
-  /* no PAR was specified in input, derive from encoded data */
-  if (dvdec->need_par) {
-    if (dvdec->PAL) {
-      if (dvdec->wide) {
-        dvdec->par_x = PAL_WIDE_PAR_X;
-        dvdec->par_y = PAL_WIDE_PAR_Y;
-      } else {
-        dvdec->par_x = PAL_NORMAL_PAR_X;
-        dvdec->par_y = PAL_NORMAL_PAR_Y;
-      }
-    } else {
-      if (dvdec->wide) {
-        dvdec->par_x = NTSC_WIDE_PAR_X;
-        dvdec->par_y = NTSC_WIDE_PAR_Y;
-      } else {
-        dvdec->par_x = NTSC_NORMAL_PAR_X;
-        dvdec->par_y = NTSC_NORMAL_PAR_Y;
-      }
-    }
-    GST_DEBUG_OBJECT (dvdec, "Inferred PAR %d/%d from video format",
-        dvdec->par_x, dvdec->par_y);
-  }
 
   /* ignoring rgb, bgr0 for now */
+
   dvdec->bpp = 2;
 
   othercaps = gst_caps_new_simple ("video/x-raw-yuv",
@@ -327,55 +261,25 @@ gst_dvdec_src_negotiate (GstDVDec * dvdec)
       "width", G_TYPE_INT, 720,
       "height", G_TYPE_INT, dvdec->height,
       "framerate", GST_TYPE_FRACTION, dvdec->framerate_numerator,
-      dvdec->framerate_denominator,
-      "pixel-aspect-ratio", GST_TYPE_FRACTION, dvdec->par_x,
-      dvdec->par_y, "interlaced", G_TYPE_BOOLEAN, dvdec->interlaced, NULL);
+      dvdec->framerate_denominator, NULL);
+  if (gotpar)
+    gst_structure_set_value (gst_caps_get_structure (othercaps, 0),
+        "pixel-aspect-ratio", par);
 
   gst_pad_set_caps (dvdec->srcpad, othercaps);
+
   gst_caps_unref (othercaps);
 
-  dvdec->src_negotiated = TRUE;
+  gst_object_unref (dvdec);
 
   return TRUE;
-}
 
-static gboolean
-gst_dvdec_sink_event (GstPad * pad, GstEvent * event)
-{
-  GstDVDec *dvdec;
-  gboolean res = TRUE;
+error:
+  {
+    gst_object_unref (dvdec);
 
-  dvdec = GST_DVDEC (gst_pad_get_parent (pad));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-      gst_segment_init (dvdec->segment, GST_FORMAT_UNDEFINED);
-      break;
-    case GST_EVENT_NEWSEGMENT:{
-      gboolean update;
-      gdouble rate, applied_rate;
-      GstFormat format;
-      gint64 start, stop, position;
-
-      gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
-          &format, &start, &stop, &position);
-
-      GST_DEBUG_OBJECT (dvdec, "Got NEWSEGMENT [%" GST_TIME_FORMAT
-          " - %" GST_TIME_FORMAT " / %" GST_TIME_FORMAT "]",
-          GST_TIME_ARGS (start), GST_TIME_ARGS (stop),
-          GST_TIME_ARGS (position));
-
-      gst_segment_set_newsegment_full (dvdec->segment, update, rate,
-          applied_rate, format, start, stop, position);
-      break;
-    }
-    default:
-      break;
+    return FALSE;
   }
-
-  res = gst_pad_push_event (dvdec->srcpad, event);
-
-  return res;
 }
 
 static GstFlowReturn
@@ -389,8 +293,6 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
   GstBuffer *outbuf;
   GstFlowReturn ret = GST_FLOW_OK;
   guint length;
-  gint64 cstart, cstop;
-  gboolean PAL, wide;
 
   dvdec = GST_DVDEC (gst_pad_get_parent (pad));
   inframe = GST_BUFFER_DATA (buf);
@@ -400,24 +302,12 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
   if (G_UNLIKELY (GST_BUFFER_SIZE (buf) < NTSC_BUFFER))
     goto wrong_size;
 
-  /* preliminary dropping. unref and return if outside of configured segment */
-  if ((dvdec->segment->format == GST_FORMAT_TIME) &&
-      (!(gst_segment_clip (dvdec->segment, GST_FORMAT_TIME,
-                  GST_BUFFER_TIMESTAMP (buf),
-                  GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf),
-                  &cstart, &cstop))))
-    goto dropping;
-
   if (G_UNLIKELY (dv_parse_header (dvdec->decoder, inframe) < 0))
     goto parse_header_error;
 
-  /* get size */
-  PAL = dv_system_50_fields (dvdec->decoder);
-  wide = dv_format_wide (dvdec->decoder);
-
   /* check the buffer is of right size after we know if we are
    * dealing with PAL or NTSC */
-  length = (PAL ? PAL_BUFFER : NTSC_BUFFER);
+  length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
   if (G_UNLIKELY (GST_BUFFER_SIZE (buf) < length))
     goto wrong_size;
 
@@ -426,23 +316,6 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
   if (dvdec->video_offset % dvdec->drop_factor != 0)
     goto skip;
 
-  /* renegotiate on change */
-  if (PAL != dvdec->PAL || wide != dvdec->wide) {
-    dvdec->src_negotiated = FALSE;
-    dvdec->PAL = PAL;
-    dvdec->wide = wide;
-  }
-
-  dvdec->height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
-
-  dvdec->interlaced = !dv_is_progressive (dvdec->decoder);
-
-  /* negotiate if not done yet */
-  if (!dvdec->src_negotiated) {
-    if (!gst_dvdec_src_negotiate (dvdec))
-      goto not_negotiated;
-  }
-
   ret =
       gst_pad_alloc_buffer_and_set_caps (dvdec->srcpad, 0,
       (720 * dvdec->height) * dvdec->bpp,
@@ -468,12 +341,7 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
   dv_decode_full_frame (dvdec->decoder, inframe,
       e_dv_color_yuv, outframe_ptrs, outframe_pitches);
 
-  GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_TFF);
-
-  GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (buf);
-  GST_BUFFER_OFFSET_END (outbuf) = GST_BUFFER_OFFSET_END (buf);
-  GST_BUFFER_TIMESTAMP (outbuf) = cstart;
-  GST_BUFFER_DURATION (outbuf) = cstop - cstart;
+  gst_buffer_stamp (outbuf, buf);
 
   ret = gst_pad_push (dvdec->srcpad, outbuf);
 
@@ -501,24 +369,11 @@ parse_header_error:
     ret = GST_FLOW_ERROR;
     goto done;
   }
-not_negotiated:
-  {
-    GST_DEBUG_OBJECT (dvdec, "could not negotiate output");
-    ret = GST_FLOW_NOT_NEGOTIATED;
-    goto done;
-  }
 no_buffer:
   {
     GST_DEBUG_OBJECT (dvdec, "could not allocate buffer");
     goto done;
   }
-
-dropping:
-  {
-    GST_DEBUG_OBJECT (dvdec,
-        "dropping buffer since it's out of the configured segment");
-    goto done;
-  }
 }
 
 static GstStateChangeReturn
@@ -536,9 +391,6 @@ gst_dvdec_change_state (GstElement * element, GstStateChange transition)
           dv_decoder_new (0, dvdec->clamp_luma, dvdec->clamp_chroma);
       dvdec->decoder->quality = qualities[dvdec->quality];
       dv_set_error_log (dvdec->decoder, NULL);
-      gst_segment_init (dvdec->segment, GST_FORMAT_UNDEFINED);
-      dvdec->src_negotiated = FALSE;
-      dvdec->sink_negotiated = FALSE;
       /* 
        * Enable this function call when libdv2 0.100 or higher is more
        * common
