@@ -25,26 +25,33 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include "esdsink.h"
 #include <esd.h>
 #include <unistd.h>
 #include <errno.h>
 
+#include <gst/gst-i18n-plugin.h>
+
+/* wtay: from my esd.h (debian unstable libesd0-dev 0.2.36-3) */
+#ifndef ESD_MAX_WRITE_SIZE
+#define ESD_MAX_WRITE_SIZE (21 * 4096)
+#endif
+
 GST_DEBUG_CATEGORY_EXTERN (esd_debug);
 #define GST_CAT_DEFAULT esd_debug
 
 /* elementfactory information */
-static GstElementDetails esdsink_details = {
-  "Esound audio sink",
-  "Sink/Audio",
-  "Plays audio to an esound server",
-  "Arwed von Merkatz <v.merkatz@gmx.net>",
-};
+static const GstElementDetails esdsink_details =
+GST_ELEMENT_DETAILS ("Esound audio sink",
+    "Sink/Audio",
+    "Plays audio to an esound server",
+    "Arwed von Merkatz <v.merkatz@gmx.net>");
 
 enum
 {
-  ARG_0,
-  ARG_HOST
+  PROP_0,
+  PROP_HOST
 };
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -67,7 +74,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 static void gst_esdsink_base_init (gpointer g_class);
 static void gst_esdsink_class_init (GstEsdSinkClass * klass);
 static void gst_esdsink_init (GstEsdSink * esdsink);
-static void gst_esdsink_dispose (GObject * object);
+static void gst_esdsink_finalize (GObject * object);
 
 static GstCaps *gst_esdsink_getcaps (GstBaseSink * bsink);
 
@@ -136,9 +143,9 @@ gst_esdsink_class_init (GstEsdSinkClass * klass)
   gstbaseaudiosink_class = (GstBaseAudioSinkClass *) klass;
   gstaudiosink_class = (GstAudioSinkClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_AUDIO_SINK);
+  parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = gst_esdsink_dispose;
+  gobject_class->finalize = gst_esdsink_finalize;
 
   gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_esdsink_getcaps);
 
@@ -153,9 +160,10 @@ gst_esdsink_class_init (GstEsdSinkClass * klass)
   gobject_class->set_property = gst_esdsink_set_property;
   gobject_class->get_property = gst_esdsink_get_property;
 
-  g_object_class_install_property (gobject_class, ARG_HOST,
-      g_param_spec_string ("host", "host", "host", NULL, G_PARAM_READWRITE));
-
+  /* default value is filled in the _init method */
+  g_object_class_install_property (gobject_class, PROP_HOST,
+      g_param_spec_string ("host", "Host",
+          "The host running the esound daemon", NULL, G_PARAM_READWRITE));
 }
 
 static void
@@ -163,17 +171,17 @@ gst_esdsink_init (GstEsdSink * esdsink)
 {
   esdsink->fd = -1;
   esdsink->ctrl_fd = -1;
-  esdsink->host = g_strdup (getenv ("ESPEAKER"));
+  esdsink->host = g_strdup (g_getenv ("ESPEAKER"));
 }
 
 static void
-gst_esdsink_dispose (GObject * object)
+gst_esdsink_finalize (GObject * object)
 {
   GstEsdSink *esdsink = GST_ESDSINK (object);
 
   g_free (esdsink->host);
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -185,32 +193,38 @@ gst_esdsink_getcaps (GstBaseSink * bsink)
   gint i;
   esd_server_info_t *server_info;
 
-  GST_DEBUG ("getcaps called");
-
   esdsink = GST_ESDSINK (bsink);
 
+  GST_DEBUG_OBJECT (esdsink, "getcaps called");
+
   pad_template = gst_static_pad_template_get (&sink_factory);
   caps = gst_caps_copy (gst_pad_template_get_caps (pad_template));
 
-  if (esdsink->ctrl_fd < 0) {
-    return caps;
-  }
+  /* no fd, we're done with the template caps */
+  if (esdsink->ctrl_fd < 0)
+    goto done;
 
+  /* get server info */
   server_info = esd_get_server_info (esdsink->ctrl_fd);
-  if (server_info) {
-    GST_DEBUG ("got server info rate: %i", server_info->rate);
+  if (!server_info)
+    goto no_info;
 
-    for (i = 0; i < caps->structs->len; i++) {
-      GstStructure *s;
+  GST_DEBUG_OBJECT (esdsink, "got server info rate: %i", server_info->rate);
 
-      s = gst_caps_get_structure (caps, i);
-      gst_structure_set (s, "rate", G_TYPE_INT, server_info->rate, NULL);
-    }
+  for (i = 0; i < caps->structs->len; i++) {
+    GstStructure *s;
 
-    esd_free_server_info (server_info);
+    s = gst_caps_get_structure (caps, i);
+    gst_structure_set (s, "rate", G_TYPE_INT, server_info->rate, NULL);
+  }
+  esd_free_server_info (server_info);
 
-    return caps;
-  } else {
+done:
+  return caps;
+
+  /* ERRORS */
+no_info:
+  {
     GST_WARNING_OBJECT (esdsink, "couldn't get server info!");
     gst_caps_unref (caps);
     return NULL;
@@ -222,16 +236,19 @@ gst_esdsink_open (GstAudioSink * asink)
 {
   GstEsdSink *esdsink = GST_ESDSINK (asink);
 
-  esdsink->ctrl_fd = esd_open_sound (esdsink->host);
+  GST_DEBUG_OBJECT (esdsink, "open");
 
+  esdsink->ctrl_fd = esd_open_sound (esdsink->host);
   if (esdsink->ctrl_fd < 0)
     goto couldnt_connect;
 
   return TRUE;
 
+  /* ERRORS */
 couldnt_connect:
   {
-    GST_ELEMENT_ERROR (esdsink, RESOURCE, OPEN_WRITE, (NULL),
+    GST_ELEMENT_ERROR (esdsink, RESOURCE, OPEN_WRITE,
+        (_("Could not establish connection to sound server")),
         ("can't open connection to esound server"));
     return FALSE;
   }
@@ -242,7 +259,10 @@ gst_esdsink_close (GstAudioSink * asink)
 {
   GstEsdSink *esdsink = GST_ESDSINK (asink);
 
+  GST_DEBUG_OBJECT (esdsink, "close");
+
   esd_close (esdsink->ctrl_fd);
+  esdsink->ctrl_fd = -1;
 
   return TRUE;
 }
@@ -251,37 +271,81 @@ static gboolean
 gst_esdsink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 {
   GstEsdSink *esdsink = GST_ESDSINK (asink);
+  esd_format_t esdformat;
 
   /* Name used by esound for this connection. */
   const char connname[] = "GStreamer";
 
+  GST_DEBUG_OBJECT (esdsink, "prepare");
+
   /* Bitmap describing audio format. */
-  esd_format_t esdformat = ESD_STREAM | ESD_PLAY;
+  esdformat = ESD_STREAM | ESD_PLAY;
 
-  if (spec->depth == 16)
-    esdformat |= ESD_BITS16;
-  else if (spec->depth == 8) {
-    esdformat |= ESD_BITS8;
+  switch (spec->depth) {
+    case 8:
+      esdformat |= ESD_BITS8;
+      break;
+    case 16:
+      esdformat |= ESD_BITS16;
+      break;
+    default:
+      goto unsupported_depth;
   }
 
-  if (spec->channels == 2)
-    esdformat |= ESD_STEREO;
-  else if (spec->channels == 1) {
-    esdformat |= ESD_MONO;
+  switch (spec->channels) {
+    case 1:
+      esdformat |= ESD_MONO;
+      break;
+    case 2:
+      esdformat |= ESD_STEREO;
+      break;
+    default:
+      goto unsupported_channels;
   }
 
-  GST_INFO ("attempting to open data connection to esound server");
+  GST_INFO_OBJECT (esdsink,
+      "attempting to open data connection to esound server");
+
   esdsink->fd =
       esd_play_stream (esdformat, spec->rate, esdsink->host, connname);
 
-  if ((esdsink->fd < 0) || (esdsink->ctrl_fd < 0)) {
-    GST_ELEMENT_ERROR (esdsink, RESOURCE, OPEN_WRITE, (NULL),
+  if ((esdsink->fd < 0) || (esdsink->ctrl_fd < 0))
+    goto cannot_open;
+
+  esdsink->rate = spec->rate;
+
+  spec->segsize = ESD_BUF_SIZE;
+  spec->segtotal = (ESD_MAX_WRITE_SIZE / spec->segsize);
+  spec->silence_sample[0] = 0;
+  spec->silence_sample[1] = 0;
+  spec->silence_sample[2] = 0;
+  spec->silence_sample[3] = 0;
+
+  GST_INFO_OBJECT (esdsink, "successfully opened connection to esound server");
+
+  return TRUE;
+
+  /* ERRORS */
+unsupported_depth:
+  {
+    GST_ELEMENT_ERROR (esdsink, STREAM, WRONG_TYPE, (NULL),
+        ("can't handle sample depth of %d, only 8 or 16 supported",
+            spec->depth));
+    return FALSE;
+  }
+unsupported_channels:
+  {
+    GST_ELEMENT_ERROR (esdsink, STREAM, WRONG_TYPE, (NULL),
+        ("can't handle %d channels, only 1 or 2 supported", spec->channels));
+    return FALSE;
+  }
+cannot_open:
+  {
+    GST_ELEMENT_ERROR (esdsink, RESOURCE, OPEN_WRITE,
+        (_("Could not establish connection to sound server")),
         ("can't open connection to esound server"));
     return FALSE;
   }
-  GST_INFO ("successfully opened connection to esound server");
-
-  return TRUE;
 }
 
 static gboolean
@@ -295,7 +359,8 @@ gst_esdsink_unprepare (GstAudioSink * asink)
   close (esdsink->fd);
   esdsink->fd = -1;
 
-  GST_INFO ("esdsink: closed sound device");
+  GST_INFO_OBJECT (esdsink, "closed sound device");
+
   return TRUE;
 }
 
@@ -313,32 +378,44 @@ gst_esdsink_write (GstAudioSink * asink, gpointer data, guint length)
 
     done = write (esdsink->fd, data, to_write);
 
-    if (done < 0) {
-      GST_ELEMENT_ERROR (esdsink, RESOURCE, WRITE,
-          ("Failed to write data to the esound daemon"), GST_ERROR_SYSTEM);
-      return 0;
-    }
+    if (done < 0)
+      goto write_error;
 
     to_write -= done;
     data += done;
   }
   return length;
+
+  /* ERRORS */
+write_error:
+  {
+    GST_ELEMENT_ERROR (esdsink, RESOURCE, WRITE,
+        ("Failed to write data to the esound daemon"), GST_ERROR_SYSTEM);
+    return 0;
+  }
 }
 
 static guint
 gst_esdsink_delay (GstAudioSink * asink)
 {
   GstEsdSink *esdsink = GST_ESDSINK (asink);
-  guint latency = esd_get_latency (esdsink->ctrl_fd);
+  guint latency;
+
+  latency = esd_get_latency (esdsink->ctrl_fd);
+
+  /* latency is measured in samples at a rate of 44100, this 
+   * cannot overflow. */
+  latency = latency * 44100LL / esdsink->rate;
+
+  GST_DEBUG_OBJECT (asink, "got latency: %u", latency);
 
-  GST_DEBUG ("got latency: %u", latency);
   return latency;
 }
 
 static void
 gst_esdsink_reset (GstAudioSink * asink)
 {
-  GST_DEBUG ("reset called");
+  GST_DEBUG_OBJECT (asink, "reset called");
 }
 
 static void
@@ -348,12 +425,9 @@ gst_esdsink_set_property (GObject * object, guint prop_id, const GValue * value,
   GstEsdSink *esdsink = GST_ESDSINK (object);
 
   switch (prop_id) {
-    case ARG_HOST:
+    case PROP_HOST:
       g_free (esdsink->host);
-      if (g_value_get_string (value) == NULL)
-        esdsink->host = NULL;
-      else
-        esdsink->host = g_strdup (g_value_get_string (value));
+      esdsink->host = g_value_dup_string (value);
       break;
     default:
       break;
@@ -367,7 +441,7 @@ gst_esdsink_get_property (GObject * object, guint prop_id, GValue * value,
   GstEsdSink *esdsink = GST_ESDSINK (object);
 
   switch (prop_id) {
-    case ARG_HOST:
+    case PROP_HOST:
       g_value_set_string (value, esdsink->host);
       break;
     default:
