@@ -23,7 +23,6 @@
 
 
 #include <gst/gst.h>
-#include <gst/base/gstadapter.h>
 
 #include <FLAC/all.h>
 
@@ -31,9 +30,9 @@ G_BEGIN_DECLS
 
 #define GST_TYPE_FLAC_DEC gst_flac_dec_get_type()
 #define GST_FLAC_DEC(obj) G_TYPE_CHECK_INSTANCE_CAST(obj, GST_TYPE_FLAC_DEC, GstFlacDec)
-#define GST_FLAC_DEC_CLASS(klass) G_TYPE_CHECK_CLASS_CAST(klass, GST_TYPE_FLAC_DEC, GstFlacDecClass)
+#define GST_FLAC_DEC_CLASS(klass) G_TYPE_CHECK_CLASS_CAST(klass, GST_TYPE_FLAC_DEC, GstFlacDec)
 #define GST_IS_FLAC_DEC(obj) G_TYPE_CHECK_INSTANCE_TYPE(obj, GST_TYPE_FLAC_DEC)
-#define GST_IS_FLAC_DEC_CLASS(klass) G_TYPE_CHECK_CLASS_TYPE(klass, GST_TYPE_FLAC_DEC)
+#define GST_IS_FLAC_DEC_CLASS(obj) G_TYPE_CHECK_CLASS_TYPE(klass, GST_TYPE_FLAC_DEC)
 
 typedef struct _GstFlacDec GstFlacDec;
 typedef struct _GstFlacDecClass GstFlacDecClass;
@@ -41,19 +40,13 @@ typedef struct _GstFlacDecClass GstFlacDecClass;
 struct _GstFlacDec {
   GstElement     element;
 
-  /* < private > */
-
-  FLAC__StreamDecoder         *decoder;
-  GstAdapter                  *adapter;
-  gboolean                     framed;
-  gboolean                     streaming;
-
-  gboolean       got_headers; /* if we've parsed the headers (unframed push mode only) */
+  FLAC__SeekableStreamDecoder *decoder;
 
   GstPad        *sinkpad;
   GstPad        *srcpad;
 
   gboolean       init;
+  gboolean       need_newsegment;
 
   guint64        offset;      /* current byte offset of input */
 
@@ -61,19 +54,8 @@ struct _GstFlacDec {
                                * don't push any buffers in the write callback
                                * until we are actually at the new position */
 
-  gboolean       eos;         /* set to TRUE if seeked after the end of file */
-
   GstSegment     segment;     /* the currently configured segment, in
                                * samples/audio frames (DEFAULT format) */
-  gboolean       running;
-  gboolean       discont;
-  GstBuffer     *pending;     /* pending buffer, produced in seek */
-  guint          pending_samples;
-  GstEvent      *close_segment;
-  GstEvent      *start_segment;
-  GstTagList    *tags;
-
-  GstFlowReturn  pull_flow;   /* last flow from pull_range */ /* STREAM_LOCK */
 
   GstFlowReturn  last_flow;   /* the last flow return received from either
                                * gst_pad_push or gst_pad_buffer_alloc */
@@ -86,8 +68,6 @@ struct _GstFlacDec {
   /* from the stream info, needed for scanning */
   guint16        min_blocksize;
   guint16        max_blocksize;
-
-  gint64         cur_granulepos; /* only used in framed mode (flac-in-ogg) */
 };
 
 struct _GstFlacDecClass {
