@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <2002> Thomas Vander Stichele <thomas@apestaart.org>
+ * Copyright (C) <2006> Jürg Billeter <j@bitron.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -45,47 +46,6 @@ gst_gconf_get_client (void)
   return _gst_gconf_client;
 }
 
-/* go through a bin, finding the one pad that is unconnected in the given
- *  * direction, and return that pad */
-static GstPad *
-gst_bin_find_unconnected_pad (GstBin * bin, GstPadDirection direction)
-{
-  GstPad *pad = NULL;
-  GList *elements = NULL;
-  const GList *pads = NULL;
-  GstElement *element = NULL;
-
-  GST_OBJECT_LOCK (bin);
-  elements = bin->children;
-  /* traverse all elements looking for unconnected pads */
-  while (elements && pad == NULL) {
-    element = GST_ELEMENT (elements->data);
-    GST_OBJECT_LOCK (element);
-    pads = element->pads;
-    while (pads) {
-      GstPad *testpad = GST_PAD (pads->data);
-
-      /* check if the direction matches */
-      if (GST_PAD_DIRECTION (testpad) == direction) {
-        GST_OBJECT_LOCK (testpad);
-        if (GST_PAD_PEER (testpad) == NULL) {
-          GST_OBJECT_UNLOCK (testpad);
-          /* found it ! */
-          pad = testpad;
-          break;
-        }
-        GST_OBJECT_UNLOCK (testpad);
-      }
-      pads = g_list_next (pads);
-    }
-    GST_OBJECT_UNLOCK (element);
-    elements = g_list_next (elements);
-  }
-  GST_OBJECT_UNLOCK (bin);
-
-  return pad;
-}
-
 /* external functions */
 
 /**
@@ -138,43 +98,6 @@ gst_gconf_set_string (const gchar * key, const gchar * value)
   g_free (full_key);
 }
 
-/**
- * gst_gconf_render_bin_from_description:
- * @description: a #gchar string describing the bin.
- *
- * Render bin from description @description.
- *
- * Returns: a #GstElement containing the rendered bin.
- */
-GstElement *
-gst_gconf_render_bin_from_description (const gchar * description)
-{
-  GstElement *bin = NULL;
-  GstPad *pad = NULL;
-  GError *error = NULL;
-  gchar *desc = NULL;
-
-  /* parse the pipeline to a bin */
-  desc = g_strdup_printf ("bin.( %s )", description);
-  bin = GST_ELEMENT (gst_parse_launch (desc, &error));
-  g_free (desc);
-  if (error) {
-    GST_ERROR ("gstgconf: error parsing pipeline %s\n%s\n",
-        description, error->message);
-    g_error_free (error);
-    return NULL;
-  }
-
-  /* find pads and ghost them if necessary */
-  if ((pad = gst_bin_find_unconnected_pad (GST_BIN (bin), GST_PAD_SRC))) {
-    gst_element_add_pad (bin, gst_ghost_pad_new ("src", pad));
-  }
-  if ((pad = gst_bin_find_unconnected_pad (GST_BIN (bin), GST_PAD_SINK))) {
-    gst_element_add_pad (bin, gst_ghost_pad_new ("sink", pad));
-  }
-  return bin;
-}
-
 /**
  * gst_gconf_render_bin_from_key:
  * @key: a #gchar string corresponding to a GConf key.
@@ -190,14 +113,23 @@ gst_gconf_render_bin_from_key (const gchar * key)
   gchar *value;
 
   value = gst_gconf_get_string (key);
-  if (value)
-    bin = gst_gconf_render_bin_from_description (value);
-  g_free (value);
+  if (value) {
+    GError *err = NULL;
+
+    bin = gst_parse_bin_from_description (value, TRUE, &err);
+    if (err) {
+      GST_ERROR ("gconf: error creating bin '%s': %s", value, err->message);
+      g_error_free (err);
+    }
+
+    g_free (value);
+  }
   return bin;
 }
 
 /**
  * gst_gconf_get_default_audio_sink:
+ * @profile: the appropriate application profile.
  *
  * Render audio output bin from GStreamer GConf key : "default/audiosink".
  * If key is invalid, the default audio sink for the platform is used
@@ -207,9 +139,30 @@ gst_gconf_render_bin_from_key (const gchar * key)
  * everything failed.
  */
 GstElement *
-gst_gconf_get_default_audio_sink (void)
+gst_gconf_get_default_audio_sink (int profile)
 {
-  GstElement *ret = gst_gconf_render_bin_from_key ("default/audiosink");
+  GstElement *ret;
+  gchar *key;
+  const gchar *profile_string;
+
+  switch (profile) {
+    case GCONF_PROFILE_SOUNDS:
+      profile_string = "";
+      break;
+    case GCONF_PROFILE_MUSIC:
+      profile_string = "music";
+      break;
+    case GCONF_PROFILE_CHAT:
+      profile_string = "chat";
+      break;
+    default:
+      g_return_val_if_reached (NULL);
+  }
+
+  key = g_strdup_printf ("default/%saudiosink", profile_string);
+
+  ret = gst_gconf_render_bin_from_key (key);
+  g_free (key);
 
   if (!ret) {
     ret = gst_element_factory_make (DEFAULT_AUDIOSINK, NULL);
@@ -235,7 +188,7 @@ gst_gconf_get_default_audio_sink (void)
 GstElement *
 gst_gconf_get_default_video_sink (void)
 {
-  GstElement *ret = gst_gconf_render_bin_from_key ("default/videosink");
+  GstElement *ret = gst_gconf_render_bin_from_key (GST_GCONF_VIDEOSINK_KEY);
 
   if (!ret) {
     ret = gst_element_factory_make (DEFAULT_VIDEOSINK, NULL);
@@ -261,7 +214,7 @@ gst_gconf_get_default_video_sink (void)
 GstElement *
 gst_gconf_get_default_audio_src (void)
 {
-  GstElement *ret = gst_gconf_render_bin_from_key ("default/audiosrc");
+  GstElement *ret = gst_gconf_render_bin_from_key (GST_GCONF_AUDIOSRC_KEY);
 
   if (!ret) {
     ret = gst_element_factory_make (DEFAULT_AUDIOSRC, NULL);
@@ -287,7 +240,7 @@ gst_gconf_get_default_audio_src (void)
 GstElement *
 gst_gconf_get_default_video_src (void)
 {
-  GstElement *ret = gst_gconf_render_bin_from_key ("default/videosrc");
+  GstElement *ret = gst_gconf_render_bin_from_key (GST_GCONF_VIDEOSRC_KEY);
 
   if (!ret) {
     ret = gst_element_factory_make (DEFAULT_VIDEOSRC, NULL);
