@@ -40,12 +40,11 @@ static void
 gst_gconf_video_src_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
-  GstElementDetails gst_gconf_video_src_details = {
-    "GConf video source",
-    "Source/Video",
-    "Video source embedding the GConf-settings for video input",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>"
-  };
+  static const GstElementDetails gst_gconf_video_src_details =
+      GST_ELEMENT_DETAILS ("GConf video source",
+      "Source/Video",
+      "Video source embedding the GConf-settings for video input",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
   GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
       GST_PAD_SRC,
       GST_PAD_ALWAYS,
@@ -103,7 +102,7 @@ gst_gconf_video_src_init (GstGConfVideoSrc * src,
   src->client = gconf_client_get_default ();
   gconf_client_add_dir (src->client, GST_GCONF_DIR,
       GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-  gconf_client_notify_add (src->client, 
+  gconf_client_notify_add (src->client,
       GST_GCONF_DIR "/" GST_GCONF_VIDEOSRC_KEY,
       cb_toggle_element, src, NULL, NULL);
 }
@@ -133,7 +132,7 @@ do_toggle_element (GstGConfVideoSrc * src)
   new_gconf_str = gst_gconf_get_string (GST_GCONF_AUDIOSRC_KEY);
   if (new_gconf_str != NULL && src->gconf_str != NULL &&
       (strlen (new_gconf_str) == 0 ||
-      strcmp (src->gconf_str, new_gconf_str) == 0)) {
+          strcmp (src->gconf_str, new_gconf_str) == 0)) {
     g_free (new_gconf_str);
     GST_DEBUG_OBJECT (src, "GConf key was updated, but it didn't change");
     return TRUE;
