@@ -1,4 +1,4 @@
-/* GStreamer Speex Encoder
+/* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  *
  * This library is free software; you can redistribute it and/or
@@ -18,54 +18,64 @@
  */
 
 
-#ifndef __GST_SPEEX_ENC_H__
-#define __GST_SPEEX_ENC_H__
+#ifndef __GST_SPEEXENC_H__
+#define __GST_SPEEXENC_H__
 
 
 #include <gst/gst.h>
-#include <gst/audio/gstaudioencoder.h>
+#include <gst/base/gstadapter.h>
 
 #include <speex/speex.h>
 #include <speex/speex_header.h>
 
 G_BEGIN_DECLS
 
-#define GST_TYPE_SPEEX_ENC \
-  (gst_speex_enc_get_type())
-#define GST_SPEEX_ENC(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_SPEEX_ENC,GstSpeexEnc))
-#define GST_SPEEX_ENC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_SPEEX_ENC,GstSpeexEncClass))
-#define GST_IS_SPEEX_ENC(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_SPEEX_ENC))
-#define GST_IS_SPEEX_ENC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_SPEEX_ENC))
+#define GST_TYPE_SPEEXENC \
+  (gst_speexenc_get_type())
+#define GST_SPEEXENC(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_SPEEXENC,GstSpeexEnc))
+#define GST_SPEEXENC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_SPEEXENC,GstSpeexEnc))
+#define GST_IS_SPEEXENC(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_SPEEXENC))
+#define GST_IS_SPEEXENC_CLASS(obj) \
+  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_SPEEXENC))
+
+#define MAX_FRAME_SIZE 2000*2
+#define MAX_FRAME_BYTES 2000
 
 typedef enum
 {
-  GST_SPEEX_ENC_MODE_AUTO,
-  GST_SPEEX_ENC_MODE_UWB,
-  GST_SPEEX_ENC_MODE_WB,
-  GST_SPEEX_ENC_MODE_NB
+  GST_SPEEXENC_MODE_AUTO,
+  GST_SPEEXENC_MODE_UWB,
+  GST_SPEEXENC_MODE_WB,
+  GST_SPEEXENC_MODE_NB,
 } GstSpeexMode;
 
 typedef struct _GstSpeexEnc GstSpeexEnc;
 typedef struct _GstSpeexEncClass GstSpeexEncClass;
 
 struct _GstSpeexEnc {
-  GstAudioEncoder   element;
+  GstElement            element;
+
+  /* pads */
+  GstPad                *sinkpad,
+                        *srcpad;
+
+  gint                  packet_count;
+  gint                  n_packets;
 
   SpeexBits             bits;
   SpeexHeader           header;
-#ifdef SPEEX_1_0
+#if SPEEX_1_0
   SpeexMode             *speex_mode;
 #else
   const SpeexMode       *speex_mode;
 #endif
   void                  *state;
-
-  /* properties */
   GstSpeexMode          mode;
+  GstAdapter            *adapter;
+
   gfloat                quality;
   gint                  bitrate;
   gboolean              vbr;
@@ -74,28 +84,40 @@ struct _GstSpeexEnc {
   gboolean              dtx;
   gint                  complexity;
   gint                  nframes;
-  gchar                 *last_message;
+
+  gint                  lookahead;
 
   gint                  channels;
   gint                  rate;
 
+  gboolean              setup;
   gboolean              header_sent;
-  GSList               *headers;
+  gboolean              eos;
+
+  guint64               samples_in;
+  guint64               bytes_out;
 
   GstTagList            *tags;
 
+  gchar                 *last_message;
+
   gint                  frame_size;
-  gint                  lookahead;
+  guint64               frameno;
 
   guint8                *comments;
   gint                  comment_len;
+
+  gfloat                input[MAX_FRAME_SIZE];
 };
 
 struct _GstSpeexEncClass {
-  GstAudioEncoderClass parent_class;
+  GstElementClass parent_class;
+
+  /* signals */
+  void (*frame_encoded) (GstElement *element);
 };
 
-GType gst_speex_enc_get_type (void);
+GType gst_speexenc_get_type(void);
 
 G_END_DECLS
 
