@@ -40,67 +40,6 @@ G_BEGIN_DECLS
 #define GST_IS_AVI_MUX_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_AVI_MUX))
 
-#define GST_AVI_INDEX_OF_INDEXES     0
-#define GST_AVI_INDEX_OF_CHUNKS      1
-
-/* this allows indexing up to 64GB avi file */
-#define GST_AVI_SUPERINDEX_COUNT    32
-
-typedef struct _gst_avi_superindex_entry {
-  guint64 offset;
-  guint32 size;
-  guint32 duration;
-} gst_avi_superindex_entry;
-
-typedef struct _GstAviPad {
-  /* do not extend, link to it */
-  /* is NULL if original sink request pad has been removed */
-  GstCollectData *collect;
-
-  /* type */
-  gboolean is_video;
-  gboolean connected;
-
-  /* chunk tag */
-  gchar *tag;
-
-  /* stream header */
-  gst_riff_strh hdr;
-
-  /* odml super indexes */
-  gst_avi_superindex_entry idx[GST_AVI_SUPERINDEX_COUNT];
-  gint idx_index;
-  gchar *idx_tag;
-} GstAviPad;
-
-typedef struct _GstAviVideoPad {
-  GstAviPad parent;
-
-  /* stream format */
-  gst_riff_strf_vids vids;
-  /* extra data */
-  GstBuffer *vids_codec_data;
-
-} GstAviVideoPad;
-
-typedef struct _GstAviAudioPad {
-  GstAviPad parent;
-
-  /* stream format */
-  gst_riff_strf_auds auds;
-  /* audio info for bps calculation */
-  guint32 audio_size;
-  guint64 audio_time;
-  /* extra data */
-  GstBuffer *auds_codec_data;
-} GstAviAudioPad;
-
-typedef struct _GstAviCollectData {
-  /* extend the CollectData */
-  GstCollectData collect;
-
-  GstAviPad      *avipad;
-} GstAviCollectData;
 
 typedef struct _GstAviMux GstAviMux;
 typedef struct _GstAviMuxClass GstAviMuxClass;
@@ -110,44 +49,42 @@ struct _GstAviMux {
 
   /* pads */
   GstPad              *srcpad;
-  /* sinkpads, video first */
-  GSList              *sinkpads;
-  /* video restricted to 1 pad */
-  guint               video_pads, audio_pads;
+  GstCollectData      *audiocollectdata;
+  gboolean             audio_pad_connected;
+  GstCollectData      *videocollectdata;
+  gboolean             video_pad_connected;
   GstCollectPads      *collect;
   GstPadEventFunction  collect_event;
 
   /* the AVI header */
-  /* still some single stream video data in mux struct */
   gst_riff_avih avi_hdr;
-  /* total number of (video) frames */
-  guint32 total_frames;
-  /* amount of total data (bytes) */
-  guint64 total_data;
-  /* amount of data (bytes) in the AVI/AVIX block;
-   * actually the movi list, so counted from and including the movi tag */
-  guint32 data_size, datax_size;
-  /* num (video) frames in the AVI/AVIX block */
-  guint32 num_frames, numx_frames;
-  /* size of hdrl list, including tag as usual */
-
-  /* total size of extra codec data */
-  guint32 codec_data_size;
-  /* state info */
+  guint32 total_frames; /* total number of frames */
+  guint64 total_data; /* amount of total data */
+  guint32 data_size, datax_size; /* amount of data (bytes) in the AVI/AVIX block */
+  guint32 num_frames, numx_frames; /* num frames in the AVI/AVIX block */
+  guint32 header_size;
   gboolean write_header;
   gboolean restart;
+  guint32 audio_size;
+  guint64 audio_time;
+
+  /* video header */
+  gst_riff_strh vids_hdr;
+  gst_riff_strf_vids vids;
+
+  /* audio header */
+  gst_riff_strh auds_hdr;
+  gst_riff_strf_auds auds;
 
   /* tags */
   GstTagList *tags;
   GstTagList *tags_snap;
+  guint32 tag_size;
 
   /* information about the AVI index ('idx') */
   gst_riff_index_entry *idx;
   gint idx_index, idx_count;
-  /* offset of *chunk* (relative to a base offset); entered in the index */
-  guint32 idx_offset;
-  /* size of idx1 chunk (including! chunk header and size bytes) */
-  guint32 idx_size;
+  guint32 idx_offset, idx_size;
 
   /* are we a big file already? */
   gboolean is_bigfile;
