@@ -16,15 +16,6 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-/**
- * SECTION:element-flxdec
- *
- * This element decodes fli/flc/flx-video into raw video
- */
-/*
- * http://www.coolutils.com/Formats/FLI
- * http://woodshole.er.usgs.gov/operations/modeling/flc.html
- */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -40,6 +31,25 @@
 GST_DEBUG_CATEGORY_STATIC (flxdec_debug);
 #define GST_CAT_DEFAULT flxdec_debug
 
+/* flx element information */
+static const GstElementDetails flxdec_details =
+GST_ELEMENT_DETAILS ("FLX audio decoder",
+    "Codec/Decoder/Audio",
+    "FLX decoder",
+    "Sepp Wijnands <mrrazz@garbage-coderz.net>, Zeeshan Ali <zeenix@gmail.com>");
+
+/* Flx signals and args */
+enum
+{
+  /* FILL ME */
+  LAST_SIGNAL
+};
+
+enum
+{
+  ARG_0
+};
+
 /* input */
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -58,7 +68,6 @@ static GstStaticPadTemplate src_video_factory = GST_STATIC_PAD_TEMPLATE ("src",
 static void gst_flxdec_class_init (GstFlxDecClass * klass);
 static void gst_flxdec_base_init (GstFlxDecClass * klass);
 static void gst_flxdec_init (GstFlxDec * flxdec);
-static void gst_flxdec_dispose (GstFlxDec * flxdec);
 
 static GstFlowReturn gst_flxdec_chain (GstPad * pad, GstBuffer * buf);
 
@@ -69,6 +78,12 @@ static gboolean gst_flxdec_src_query_handler (GstPad * pad, GstQuery * query);
 static gboolean gst_flxdec_src_event_handler (GstPad * pad, GstEvent * event);
 static gboolean gst_flxdec_sink_event_handler (GstPad * pad, GstEvent * event);
 
+static void gst_flxdec_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_flxdec_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+
+
 static void flx_decode_color (GstFlxDec *, guchar *, guchar *, gint);
 static void flx_decode_brun (GstFlxDec *, guchar *, guchar *);
 static void flx_decode_delta_fli (GstFlxDec *, guchar *, guchar *);
@@ -107,14 +122,11 @@ gst_flxdec_base_init (GstFlxDecClass * klass)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details_simple (gstelement_class, "FLX video decoder",
-      "Codec/Decoder/Video",
-      "FLC/FLI/FLX video decoder",
-      "Sepp Wijnands <mrrazz@garbage-coderz.net>, Zeeshan Ali <zeenix@gmail.com>");
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &sink_factory);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &src_video_factory);
+  gst_element_class_set_details (gstelement_class, &flxdec_details);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_video_factory));
 }
 
 static void
@@ -128,10 +140,11 @@ gst_flxdec_class_init (GstFlxDecClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = (GObjectFinalizeFunc) gst_flxdec_dispose;
-
   GST_DEBUG_CATEGORY_INIT (flxdec_debug, "flxdec", 0, "FLX video decoder");
 
+  gobject_class->set_property = gst_flxdec_set_property;
+  gobject_class->get_property = gst_flxdec_get_property;
+
   gstelement_class->change_state = gst_flxdec_change_state;
 }
 
@@ -156,17 +169,6 @@ gst_flxdec_init (GstFlxDec * flxdec)
   flxdec->adapter = gst_adapter_new ();
 }
 
-static void
-gst_flxdec_dispose (GstFlxDec * flxdec)
-{
-  if (flxdec->adapter) {
-    g_object_unref (flxdec->adapter);
-    flxdec->adapter = NULL;
-  }
-
-  G_OBJECT_CLASS (parent_class)->dispose ((GObject *) flxdec);
-}
-
 static gboolean
 gst_flxdec_src_query_handler (GstPad * pad, GstQuery * query)
 {
@@ -355,7 +357,7 @@ flx_decode_brun (GstFlxDec * flxdec, guchar * data, guchar * dest)
 static void
 flx_decode_delta_fli (GstFlxDec * flxdec, guchar * data, guchar * dest)
 {
-  gulong count, packets, lines, start_line;
+  gulong count, packets, lines, start_line, start_l;
   guchar *start_p, x;
 
   g_return_if_fail (flxdec != NULL);
@@ -372,6 +374,7 @@ flx_decode_delta_fli (GstFlxDec * flxdec, guchar * data, guchar * dest)
   /* start position of delta */
   dest += (flxdec->hdr.width * start_line);
   start_p = dest;
+  start_l = lines;
 
   while (lines--) {
     /* packet count */
@@ -676,6 +679,37 @@ gst_flxdec_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+static void
+gst_flxdec_set_property (GObject * object, guint prop_id, const GValue * value,
+    GParamSpec * pspec)
+{
+  GstFlxDec *flxdec;
+
+  g_return_if_fail (GST_IS_FLXDEC (object));
+  flxdec = GST_FLXDEC (object);
+
+  switch (prop_id) {
+    default:
+      break;
+  }
+}
+
+static void
+gst_flxdec_get_property (GObject * object, guint prop_id, GValue * value,
+    GParamSpec * pspec)
+{
+  GstFlxDec *flxdec;
+
+  g_return_if_fail (GST_IS_FLXDEC (object));
+  flxdec = GST_FLXDEC (object);
+
+  switch (prop_id) {
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -686,5 +720,5 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "flxdec",
-    "FLC/FLI/FLX video decoder",
+    "FLX video decoder",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
