@@ -53,10 +53,9 @@ gst_matroska_track_init_video_context (GstMatroskaTrackContext ** p_context)
   video_context->display_height = 0;
   video_context->pixel_width = 0;
   video_context->pixel_height = 0;
+  video_context->eye_mode = 0;
   video_context->asr_mode = 0;
   video_context->fourcc = 0;
-  video_context->default_fps = 0.0;
-  video_context->earliest_time = GST_CLOCK_TIME_NONE;
   return TRUE;
 }
 
@@ -106,13 +105,27 @@ gst_matroska_track_init_subtitle_context (GstMatroskaTrackContext ** p_context)
   *p_context = (GstMatroskaTrackContext *) subtitle_context;
 
   (*p_context)->type = GST_MATROSKA_TRACK_TYPE_SUBTITLE;
-  subtitle_context->invalid_utf8 = FALSE;
-  subtitle_context->seen_markup_tag = FALSE;
   return TRUE;
 }
 
-void
-gst_matroska_register_tags (void)
+gboolean
+gst_matroska_track_init_complex_context (GstMatroskaTrackContext ** p_context)
 {
-  /* TODO: register other custom tags */
+  GstMatroskaTrackComplexContext *complex_context;
+
+  g_assert (p_context != NULL && *p_context != NULL);
+
+  /* already set up? (track info might come before track type) */
+  if ((*p_context)->type == GST_MATROSKA_TRACK_TYPE_COMPLEX)
+    return TRUE;
+
+  /* it better not have been set up as some other track type ... */
+  if ((*p_context)->type != 0) {
+    g_return_val_if_reached (FALSE);
+  }
+
+  complex_context = g_renew (GstMatroskaTrackComplexContext, *p_context, 1);
+  *p_context = (GstMatroskaTrackContext *) complex_context;
+  (*p_context)->type = GST_MATROSKA_TRACK_TYPE_COMPLEX;
+  return TRUE;
 }
