@@ -33,11 +33,18 @@
 GST_DEBUG_CATEGORY_STATIC (matroskamux_debug);
 #define GST_CAT_DEFAULT matroskamux_debug
 
+enum
+{
+  /* FILL ME */
+  LAST_SIGNAL
+};
+
 enum
 {
   ARG_0,
   ARG_WRITING_APP,
   ARG_MATROSKA_VERSION
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
@@ -51,10 +58,6 @@ static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
   "height = (int) [ 16, 4096 ], " \
   "framerate = (fraction) [ 0, MAX ]"
 
-#define COMMON_VIDEO_CAPS_NO_FRAMERATE \
-  "width = (int) [ 16, 4096 ], " \
-  "height = (int) [ 16, 4096 ] "
-
 static GstStaticPadTemplate videosink_templ =
     GST_STATIC_PAD_TEMPLATE ("video_%d",
     GST_PAD_SINK,
@@ -78,9 +81,6 @@ static GstStaticPadTemplate videosink_templ =
         "video/x-msmpeg, "
         COMMON_VIDEO_CAPS "; "
         "image/jpeg, "
-        COMMON_VIDEO_CAPS_NO_FRAMERATE "; "
-        "video/x-theora; "
-        "video/x-dirac, "
         COMMON_VIDEO_CAPS "; "
         "video/x-raw-yuv, "
         "format = (fourcc) { YUY2, I420 }, " COMMON_VIDEO_CAPS)
@@ -109,15 +109,10 @@ static GstStaticPadTemplate audiosink_templ =
         "audio/x-vorbis, "
         COMMON_AUDIO_CAPS "; "
         "audio/x-raw-int, "
-        "width = (int) 8, "
-        "depth = (int) 8, "
-        "signed = (boolean) false, "
-        COMMON_AUDIO_CAPS ";"
-        "audio/x-raw-int, "
-        "width = (int) 16, "
-        "depth = (int) 16, "
+        "width = (int) { 8, 16, 24 }, "
+        "depth = (int) { 8, 16, 24 }, "
         "endianness = (int) { BIG_ENDIAN, LITTLE_ENDIAN }, "
-        "signed = (boolean) true, "
+        "signed = (boolean) { true, false }, "
         COMMON_AUDIO_CAPS ";"
         "audio/x-tta, "
         "width = (int) { 8, 16, 24 }, "
@@ -132,10 +127,15 @@ GST_STATIC_PAD_TEMPLATE ("subtitle_%d",
 
 static GArray *used_uids;
 
-static void gst_matroska_mux_add_interfaces (GType type);
+static void
+_do_init (GType matroskamux_type)
+{
+  GST_DEBUG_CATEGORY_INIT (matroskamux_debug, "matroskamux", 0,
+      "Matroska muxer");
+}
 
 GST_BOILERPLATE_FULL (GstMatroskaMux, gst_matroska_mux, GstElement,
-    GST_TYPE_ELEMENT, gst_matroska_mux_add_interfaces);
+    GST_TYPE_ELEMENT, _do_init);
 
 /* Matroska muxer destructor */
 static void gst_matroska_mux_finalize (GObject * object);
@@ -149,7 +149,6 @@ static gboolean gst_matroska_mux_handle_src_event (GstPad * pad,
     GstEvent * event);
 static GstPad *gst_matroska_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
-static void gst_matroska_mux_release_pad (GstElement * element, GstPad * pad);
 
 /* gst internal change state handler */
 static GstStateChangeReturn
@@ -167,18 +166,7 @@ static void gst_matroska_mux_reset (GstElement * element);
 /* uid generation */
 static guint32 gst_matroska_mux_create_uid ();
 
-static gboolean theora_streamheader_to_codecdata (const GValue * streamheader,
-    GstMatroskaTrackContext * context);
-static gboolean vorbis_streamheader_to_codecdata (const GValue * streamheader,
-    GstMatroskaTrackContext * context);
-
-static void
-gst_matroska_mux_add_interfaces (GType type)
-{
-  static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
-
-  g_type_add_interface_static (type, GST_TYPE_TAG_SETTER, &tag_setter_info);
-}
+/*static guint gst_matroska_mux_signals[LAST_SIGNAL] = { 0 };*/
 
 static void
 gst_matroska_mux_base_init (gpointer g_class)
@@ -199,9 +187,6 @@ gst_matroska_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
   gst_element_class_set_details (element_class, &gst_matroska_mux_details);
-
-  GST_DEBUG_CATEGORY_INIT (matroskamux_debug, "matroskamux", 0,
-      "Matroska muxer");
 }
 
 static void
@@ -227,12 +212,8 @@ gst_matroska_mux_class_init (GstMatroskaMuxClass * klass)
           "This parameter determines what matroska features can be used.",
           1, 2, 1, G_PARAM_READWRITE));
 
-  gstelement_class->change_state =
-      GST_DEBUG_FUNCPTR (gst_matroska_mux_change_state);
-  gstelement_class->request_new_pad =
-      GST_DEBUG_FUNCPTR (gst_matroska_mux_request_new_pad);
-  gstelement_class->release_pad =
-      GST_DEBUG_FUNCPTR (gst_matroska_mux_release_pad);
+  gstelement_class->change_state = gst_matroska_mux_change_state;
+  gstelement_class->request_new_pad = gst_matroska_mux_request_new_pad;
 }
 
 
@@ -319,33 +300,6 @@ gst_matroska_mux_create_uid (void)
   return uid;
 }
 
-/**
- * gst_matroska_pad_free:
- * @collect_pad: the #GstMatroskaPad
- *
- * Release resources of a matroska collect pad.
- */
-static void
-gst_matroska_pad_free (GstMatroskaPad * collect_pad)
-{
-  /* free track information */
-  if (collect_pad->track != NULL) {
-    g_free (collect_pad->track->codec_id);
-    g_free (collect_pad->track->codec_name);
-    g_free (collect_pad->track->name);
-    g_free (collect_pad->track->language);
-    g_free (collect_pad->track->codec_priv);
-    g_free (collect_pad->track);
-    collect_pad->track = NULL;
-  }
-
-  /* free cached buffer */
-  if (collect_pad->buffer != NULL) {
-    gst_buffer_unref (collect_pad->buffer);
-    collect_pad->buffer = NULL;
-  }
-}
-
 
 /**
  * gst_matroska_mux_reset:
@@ -373,8 +327,22 @@ gst_matroska_mux_reset (GstElement * element)
     collect_pad = (GstMatroskaPad *) walk->data;
     thepad = collect_pad->collect.pad;
 
-    /* free collect pad resources */
-    gst_matroska_pad_free (collect_pad);
+    /* free track information */
+    if (collect_pad->track != NULL) {
+      g_free (collect_pad->track->codec_id);
+      g_free (collect_pad->track->codec_name);
+      g_free (collect_pad->track->name);
+      g_free (collect_pad->track->language);
+      g_free (collect_pad->track->codec_priv);
+      g_free (collect_pad->track);
+      collect_pad->track = NULL;
+    }
+
+    /* free cached buffer */
+    if (collect_pad->buffer != NULL) {
+      gst_buffer_unref (collect_pad->buffer);
+      collect_pad->buffer = NULL;
+    }
 
     /* remove from collectpads */
     gst_collect_pads_remove_pad (mux->collect, thepad);
@@ -415,12 +383,6 @@ gst_matroska_mux_reset (GstElement * element)
   mux->num_meta_indexes = 0;
   g_free (mux->meta_index);
   mux->meta_index = NULL;
-
-  /* reset tags */
-  if (mux->tags) {
-    gst_tag_list_free (mux->tags);
-    mux->tags = NULL;
-  }
 }
 
 /**
@@ -450,56 +412,6 @@ gst_matroska_mux_handle_src_event (GstPad * pad, GstEvent * event)
   return gst_pad_event_default (pad, event);
 }
 
-/**
- * gst_matroska_mux_handle_sink_event:
- * @pad: Pad which received the event.
- * @event: Received event.
- *
- * handle events - informational ones like tags
- *
- * Returns: #TRUE on success.
- */
-static gboolean
-gst_matroska_mux_handle_sink_event (GstPad * pad, GstEvent * event)
-{
-  GstMatroskaTrackContext *context;
-  GstMatroskaPad *collect_pad;
-  GstMatroskaMux *mux;
-  GstTagList *list;
-  gboolean ret;
-
-  mux = GST_MATROSKA_MUX (gst_pad_get_parent (pad));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_TAG:
-      gst_event_parse_tag (event, &list);
-      collect_pad = (GstMatroskaPad *) gst_pad_get_element_private (pad);
-      g_assert (collect_pad);
-      context = collect_pad->track;
-      g_assert (context);
-      /* FIXME ?
-       * strictly speaking, the incoming language code may only be 639-1, so not
-       * 639-2 according to matroska specs, but it will have to do for now */
-      gst_tag_list_get_string (list, GST_TAG_LANGUAGE_CODE, &context->language);
-
-      if (mux->tags) {
-        gst_tag_list_insert (mux->tags, list, GST_TAG_MERGE_PREPEND);
-      } else {
-        mux->tags = gst_tag_list_copy (list);
-      }
-
-      break;
-    default:
-      break;
-  }
-
-  /* now GstCollectPads can take care of the rest, e.g. EOS */
-  ret = mux->collect_event (pad, event);
-  gst_object_unref (mux);
-
-  return ret;
-}
-
 
 /**
  * gst_matroska_mux_video_pad_setcaps:
@@ -515,14 +427,11 @@ gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstMatroskaTrackContext *context = NULL;
   GstMatroskaTrackVideoContext *videocontext;
-  GstMatroskaMux *mux;
   GstMatroskaPad *collect_pad;
-  GstStructure *structure;
   const gchar *mimetype;
   gint width, height, pixel_width, pixel_height;
-  gint fps_d, fps_n;
-
-  mux = GST_MATROSKA_MUX (GST_PAD_PARENT (pad));
+  const GValue *framerate;
+  GstStructure *structure;
 
   /* find context */
   collect_pad = (GstMatroskaPad *) gst_pad_get_element_private (pad);
@@ -537,24 +446,19 @@ gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
 
   mimetype = gst_structure_get_name (structure);
 
-  if (!strcmp (mimetype, "video/x-theora")) {
-    /* we'll extract the details later from the theora identification header */
-    goto skip_details;
-  }
-
   /* get general properties */
   gst_structure_get_int (structure, "width", &width);
   gst_structure_get_int (structure, "height", &height);
+  framerate = gst_structure_get_value (structure, "framerate");
+  if (framerate == NULL || !GST_VALUE_HOLDS_FRACTION (framerate))
+    return FALSE;
+
   videocontext->pixel_width = width;
   videocontext->pixel_height = height;
-  if (gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d)) {
-    context->default_duration =
-        gst_util_uint64_scale_int (GST_SECOND, fps_d, fps_n);
-    GST_LOG_OBJECT (pad, "default duration = %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (context->default_duration));
-  } else {
-    context->default_duration = 0;
-  }
+  context->default_duration = gst_util_uint64_scale_int (GST_SECOND,
+      gst_value_get_fraction_denominator (framerate),
+      gst_value_get_fraction_numerator (framerate));
+
   if (gst_structure_get_fraction (structure, "pixel-aspect-ratio",
           &pixel_width, &pixel_height)) {
     if (pixel_width > pixel_height) {
@@ -572,8 +476,6 @@ gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
     videocontext->display_height = 0;
   }
 
-skip_details:
-
   videocontext->asr_mode = GST_MATROSKA_ASPECT_RATIO_MODE_FREE;
   videocontext->eye_mode = GST_MATROSKA_EYE_MODE_MONO;
   videocontext->fourcc = 0;
@@ -679,28 +581,6 @@ skip_details:
       context->codec_priv_size = priv_data_size;
     }
 
-    return TRUE;
-  } else if (!strcmp (mimetype, "video/x-theora")) {
-    const GValue *streamheader;
-
-    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_THEORA);
-
-    if (context->codec_priv != NULL) {
-      g_free (context->codec_priv);
-      context->codec_priv = NULL;
-      context->codec_priv_size = 0;
-    }
-
-    streamheader = gst_structure_get_value (structure, "streamheader");
-    if (!theora_streamheader_to_codecdata (streamheader, context)) {
-      GST_ELEMENT_ERROR (mux, STREAM, MUX, (NULL),
-          ("theora stream headers missing or malformed"));
-      return FALSE;
-    }
-    return TRUE;
-  } else if (!strcmp (mimetype, "video/x-dirac")) {
-    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_DIRAC);
-
     return TRUE;
   } else if (!strcmp (mimetype, "video/mpeg")) {
     gint mpegversion;
@@ -730,182 +610,6 @@ skip_details:
   return FALSE;
 }
 
-static gboolean
-xiph3_streamheader_to_codecdata (const GValue * streamheader,
-    GstMatroskaTrackContext * context, GstBuffer ** p_buf0)
-{
-  GstBuffer *buf[3];
-  GArray *bufarr;
-  guint8 *priv_data;
-  guint i, offset, priv_data_size;
-
-  if (streamheader == NULL)
-    goto no_stream_headers;
-
-  if (G_VALUE_TYPE (streamheader) != GST_TYPE_ARRAY)
-    goto wrong_type;
-
-  bufarr = g_value_peek_pointer (streamheader);
-  if (bufarr->len != 3)
-    goto wrong_count;
-
-  context->xiph_headers_to_skip = bufarr->len;
-
-  for (i = 0; i < 3; i++) {
-    GValue *bufval = &g_array_index (bufarr, GValue, i);
-
-    if (G_VALUE_TYPE (bufval) != GST_TYPE_BUFFER)
-      goto wrong_content_type;
-
-    buf[i] = g_value_peek_pointer (bufval);
-  }
-
-  priv_data_size = 1;
-  priv_data_size += GST_BUFFER_SIZE (buf[0]) / 0xff + 1;
-  priv_data_size += GST_BUFFER_SIZE (buf[1]) / 0xff + 1;
-
-  for (i = 0; i < 3; ++i) {
-    priv_data_size += GST_BUFFER_SIZE (buf[i]);
-  }
-
-  priv_data = g_malloc0 (priv_data_size);
-
-  priv_data[0] = 2;
-  offset = 1;
-
-  for (i = 0; i < GST_BUFFER_SIZE (buf[0]) / 0xff; ++i) {
-    priv_data[offset++] = 0xff;
-  }
-  priv_data[offset++] = GST_BUFFER_SIZE (buf[0]) % 0xff;
-
-  for (i = 0; i < GST_BUFFER_SIZE (buf[1]) / 0xff; ++i) {
-    priv_data[offset++] = 0xff;
-  }
-  priv_data[offset++] = GST_BUFFER_SIZE (buf[1]) % 0xff;
-
-  for (i = 0; i < 3; ++i) {
-    memcpy (priv_data + offset, GST_BUFFER_DATA (buf[i]),
-        GST_BUFFER_SIZE (buf[i]));
-    offset += GST_BUFFER_SIZE (buf[i]);
-  }
-
-  context->codec_priv = priv_data;
-  context->codec_priv_size = priv_data_size;
-
-  if (p_buf0)
-    *p_buf0 = gst_buffer_ref (buf[0]);
-
-  return TRUE;
-
-/* ERRORS */
-no_stream_headers:
-  {
-    GST_WARNING ("required streamheaders missing in sink caps!");
-    return FALSE;
-  }
-wrong_type:
-  {
-    GST_WARNING ("streamheaders are not a GST_TYPE_ARRAY, but a %s",
-        G_VALUE_TYPE_NAME (streamheader));
-    return FALSE;
-  }
-wrong_count:
-  {
-    GST_WARNING ("got %u streamheaders, not 3 as expected", bufarr->len);
-    return FALSE;
-  }
-wrong_content_type:
-  {
-    GST_WARNING ("streamheaders array does not contain GstBuffers");
-    return FALSE;
-  }
-}
-
-static gboolean
-vorbis_streamheader_to_codecdata (const GValue * streamheader,
-    GstMatroskaTrackContext * context)
-{
-  GstBuffer *buf0 = NULL;
-
-  if (!xiph3_streamheader_to_codecdata (streamheader, context, &buf0))
-    return FALSE;
-
-  if (buf0 == NULL || GST_BUFFER_SIZE (buf0) < 1 + 6 + 4) {
-    GST_WARNING ("First vorbis header too small, ignoring");
-  } else {
-    if (memcmp (GST_BUFFER_DATA (buf0) + 1, "vorbis", 6) == 0) {
-      GstMatroskaTrackAudioContext *audiocontext;
-      guint8 *hdr;
-
-      hdr = GST_BUFFER_DATA (buf0) + 1 + 6 + 4;
-      audiocontext = (GstMatroskaTrackAudioContext *) context;
-      audiocontext->channels = GST_READ_UINT8 (hdr);
-      audiocontext->samplerate = GST_READ_UINT32_LE (hdr + 1);
-    }
-  }
-
-  if (buf0)
-    gst_buffer_unref (buf0);
-
-  return TRUE;
-}
-
-static gboolean
-theora_streamheader_to_codecdata (const GValue * streamheader,
-    GstMatroskaTrackContext * context)
-{
-  GstBuffer *buf0 = NULL;
-
-  if (!xiph3_streamheader_to_codecdata (streamheader, context, &buf0))
-    return FALSE;
-
-  if (buf0 == NULL || GST_BUFFER_SIZE (buf0) < 1 + 6 + 26) {
-    GST_WARNING ("First theora header too small, ignoring");
-  } else if (memcmp (GST_BUFFER_DATA (buf0), "\200theora\003\002", 9) != 0) {
-    GST_WARNING ("First header not a theora identification header, ignoring");
-  } else {
-    GstMatroskaTrackVideoContext *videocontext;
-    guint fps_num, fps_denom, par_num, par_denom;
-    guint8 *hdr;
-
-    hdr = GST_BUFFER_DATA (buf0) + 1 + 6 + 3 + 2 + 2;
-
-    videocontext = (GstMatroskaTrackVideoContext *) context;
-    videocontext->pixel_width = GST_READ_UINT32_BE (hdr) >> 8;
-    videocontext->pixel_height = GST_READ_UINT32_BE (hdr + 3) >> 8;
-    hdr += 3 + 3 + 1 + 1;
-    fps_num = GST_READ_UINT32_BE (hdr);
-    fps_denom = GST_READ_UINT32_BE (hdr + 4);
-    context->default_duration = gst_util_uint64_scale_int (GST_SECOND,
-        fps_denom, fps_num);
-    hdr += 4 + 4;
-    par_num = GST_READ_UINT32_BE (hdr) >> 8;
-    par_denom = GST_READ_UINT32_BE (hdr + 3) >> 8;
-    if (par_num > 0 && par_num > 0) {
-      if (par_num > par_denom) {
-        videocontext->display_width =
-            videocontext->pixel_width * par_num / par_denom;
-        videocontext->display_height = videocontext->pixel_height;
-      } else if (par_num < par_denom) {
-        videocontext->display_width = videocontext->pixel_width;
-        videocontext->display_height =
-            videocontext->pixel_height * par_denom / par_num;
-      } else {
-        videocontext->display_width = 0;
-        videocontext->display_height = 0;
-      }
-    } else {
-      videocontext->display_width = 0;
-      videocontext->display_height = 0;
-    }
-    hdr += 3 + 3;
-  }
-
-  if (buf0)
-    gst_buffer_unref (buf0);
-
-  return TRUE;
-}
 
 /**
  * gst_matroska_mux_audio_pad_setcaps:
@@ -921,14 +625,11 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstMatroskaTrackContext *context = NULL;
   GstMatroskaTrackAudioContext *audiocontext;
-  GstMatroskaMux *mux;
   GstMatroskaPad *collect_pad;
   const gchar *mimetype;
   gint samplerate = 0, channels = 0;
   GstStructure *structure;
 
-  mux = GST_MATROSKA_MUX (GST_PAD_PARENT (pad));
-
   /* find context */
   collect_pad = (GstMatroskaPad *) gst_pad_get_element_private (pad);
   g_assert (collect_pad);
@@ -994,29 +695,15 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
     gint endianness, width, depth;
     gboolean signedness;
 
-    if (!gst_structure_get_int (structure, "width", &width) ||
+    if (!gst_structure_get_int (structure, "endianness", &endianness) ||
+        !gst_structure_get_int (structure, "width", &width) ||
         !gst_structure_get_int (structure, "depth", &depth) ||
-        !gst_structure_get_boolean (structure, "signed", &signedness)) {
-      GST_DEBUG_OBJECT (mux, "broken caps, width/depth/signed field missing");
+        !gst_structure_get_int (structure, "signed", &signedness))
       return FALSE;
-    }
 
-    if (depth > 8 &&
-        !gst_structure_get_int (structure, "endianness", &endianness)) {
-      GST_DEBUG_OBJECT (mux, "broken caps, no endianness specified");
+    if (width != depth ||
+        (width == 8 && signedness) || (width == 16 && !signedness))
       return FALSE;
-    }
-
-    if (width != depth) {
-      GST_DEBUG_OBJECT (mux, "width must be same as depth!");
-      return FALSE;
-    }
-
-    /* where is this spec'ed out? (tpm) */
-    if ((width == 8 && signedness) || (width == 16 && !signedness)) {
-      GST_DEBUG_OBJECT (mux, "8-bit PCM must be unsigned, 16-bit PCM signed");
-      return FALSE;
-    }
 
     audiocontext->bitdepth = depth;
     if (endianness == G_BIG_ENDIAN)
@@ -1029,6 +716,8 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
     /* FIXME: endianness is undefined */
   } else if (!strcmp (mimetype, "audio/x-vorbis")) {
     const GValue *streamheader;
+    guint8 *priv_data = NULL;
+    guint priv_data_size = 0;
 
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_VORBIS);
 
@@ -1037,14 +726,81 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
       context->codec_priv = NULL;
       context->codec_priv_size = 0;
     }
-
     streamheader = gst_structure_get_value (structure, "streamheader");
-    if (!vorbis_streamheader_to_codecdata (streamheader, context)) {
-      GST_ELEMENT_ERROR (mux, STREAM, MUX, (NULL),
-          ("vorbis stream headers missing or malformed"));
-      return FALSE;
+
+    if (streamheader != NULL) {
+      if (G_VALUE_TYPE (streamheader) == GST_TYPE_ARRAY) {
+        GArray *bufarr = g_value_peek_pointer (streamheader);
+        gint i;
+        gint offset;
+
+        if (bufarr->len == 3) {
+          GstBuffer *buf[3];
+
+          for (i = 0; i < bufarr->len; i++) {
+            GValue *bufval = &g_array_index (bufarr, GValue, i);
+
+            if (G_VALUE_TYPE (bufval) == GST_TYPE_BUFFER) {
+              buf[i] = g_value_peek_pointer (bufval);
+            }
+          }
+
+          priv_data_size = 1;
+          priv_data_size += GST_BUFFER_SIZE (buf[0]) / 0xff + 1;
+          priv_data_size += GST_BUFFER_SIZE (buf[1]) / 0xff + 1;
+
+          for (i = 0; i < 3; ++i) {
+            priv_data_size += GST_BUFFER_SIZE (buf[i]);
+          }
+
+          priv_data = g_malloc0 (priv_data_size);
+
+          priv_data[0] = 2;
+          offset = 1;
+
+          for (i = 0; i < GST_BUFFER_SIZE (buf[0]) / 0xff; ++i) {
+            priv_data[offset++] = 0xff;
+          }
+          priv_data[offset++] = GST_BUFFER_SIZE (buf[0]) % 0xff;
+
+          for (i = 0; i < GST_BUFFER_SIZE (buf[1]) / 0xff; ++i) {
+            priv_data[offset++] = 0xff;
+          }
+          priv_data[offset++] = GST_BUFFER_SIZE (buf[1]) % 0xff;
+
+          for (i = 0; i < 3; ++i) {
+            memcpy (priv_data + offset, GST_BUFFER_DATA (buf[i]),
+                GST_BUFFER_SIZE (buf[i]));
+            offset += GST_BUFFER_SIZE (buf[i]);
+          }
+
+          if (GST_BUFFER_SIZE (buf[0]) < 1 + 6 + 4) {
+            GST_WARNING ("First vorbis header too small, ignoring");
+          } else {
+            if (memcmp (GST_BUFFER_DATA (buf[0]) + 1, "vorbis", 6) == 0) {
+              guint8 *hdr = GST_BUFFER_DATA (buf[0]) + 1 + 6 + 4;
+
+              audiocontext->channels = GST_READ_UINT8 (hdr);
+              audiocontext->samplerate = GST_READ_UINT32_LE (hdr + 1);
+            }
+          }
+        } else {
+          GST_WARNING ("Vorbis header does not contain "
+              "three buffers (found %d buffers), Ignoring.", bufarr->len);
+        }
+      }
     }
+
+    if (priv_data == NULL) {
+      GST_WARNING ("Could not write Vorbis header into codec private data. "
+          "You will probably not be able to play the stream.");
+    }
+
+    context->codec_priv_size = priv_data_size;
+    context->codec_priv = (gpointer) priv_data;
+
     return TRUE;
+
   } else if (!strcmp (mimetype, "audio/x-ac3")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AC3);
 
@@ -1141,71 +897,13 @@ gst_matroska_mux_request_new_pad (GstElement * element,
   context->flags = GST_MATROSKA_TRACK_ENABLED | GST_MATROSKA_TRACK_DEFAULT;
   collect_pad->track = context;
   collect_pad->buffer = NULL;
-  collect_pad->start_ts = GST_CLOCK_TIME_NONE;
-  collect_pad->end_ts = GST_CLOCK_TIME_NONE;
-
-  /* FIXME: hacked way to override/extend the event function of
-   * GstCollectPads; because it sets its own event function giving the
-   * element no access to events.
-   * TODO GstCollectPads should really give its 'users' a clean chance to
-   * properly handle events that are not meant for collectpads itself.
-   * Perhaps a callback or so, though rejected (?) in #340060.
-   * This would allow (clean) transcoding of info from demuxer/streams
-   * to another muxer */
-  mux->collect_event = (GstPadEventFunction) GST_PAD_EVENTFUNC (newpad);
-  gst_pad_set_event_function (newpad,
-      GST_DEBUG_FUNCPTR (gst_matroska_mux_handle_sink_event));
 
   gst_pad_set_setcaps_function (newpad, setcapsfunc);
-  gst_pad_set_active (newpad, TRUE);
   gst_element_add_pad (element, newpad);
 
   return newpad;
 }
 
-/**
- * gst_matroska_mux_release_pad:
- * @element: #GstMatroskaMux.
- * @pad: Pad to release.
- *
- * Release a previously requested pad.
-*/
-static void
-gst_matroska_mux_release_pad (GstElement * element, GstPad * pad)
-{
-  GstMatroskaMux *mux;
-  GSList *walk;
-
-  mux = GST_MATROSKA_MUX (GST_PAD_PARENT (pad));
-
-  for (walk = mux->collect->data; walk; walk = g_slist_next (walk)) {
-    GstCollectData *cdata = (GstCollectData *) walk->data;
-    GstMatroskaPad *collect_pad = (GstMatroskaPad *) cdata;
-
-    if (cdata->pad == pad) {
-      GstClockTime min_dur;     /* observed minimum duration */
-
-      if (GST_CLOCK_TIME_IS_VALID (collect_pad->start_ts) &&
-          GST_CLOCK_TIME_IS_VALID (collect_pad->end_ts)) {
-        min_dur = GST_CLOCK_DIFF (collect_pad->start_ts, collect_pad->end_ts);
-        if (collect_pad->duration < min_dur)
-          collect_pad->duration = min_dur;
-      }
-
-      if (GST_CLOCK_TIME_IS_VALID (collect_pad->duration) &&
-          mux->duration < collect_pad->duration)
-        mux->duration = collect_pad->duration;
-
-      gst_matroska_pad_free (collect_pad);
-      gst_collect_pads_remove_pad (mux->collect, pad);
-      gst_element_remove_pad (element, pad);
-      return;
-    }
-  }
-
-  g_warning ("%s: unknown pad %s", GST_FUNCTION, GST_PAD_NAME (pad));
-}
-
 
 /**
  * gst_matroska_mux_track_header:
@@ -1231,10 +929,6 @@ gst_matroska_mux_track_header (GstMatroskaMux * mux,
     gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKDEFAULTDURATION,
         context->default_duration);
   }
-  if (context->language) {
-    gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_TRACKLANGUAGE,
-        context->language);
-  }
 
   /* type-specific stuff */
   switch (context->type) {
@@ -1318,14 +1012,16 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
     GST_MATROSKA_ID_TRACKS,
     GST_MATROSKA_ID_CUES,
     GST_MATROSKA_ID_SEEKHEAD,
+#if 0
     GST_MATROSKA_ID_TAGS,
+#endif
     0
   };
   guint64 master, child;
   GSList *collected;
   int i;
   guint tracknum = 1;
-  GstClockTime duration = 0;
+  gdouble duration = 0;
   guint32 *segment_uid = (guint32 *) g_malloc (16);
   GRand *rand = g_rand_new ();
   GTimeVal time = { 0, 0 };
@@ -1369,26 +1065,26 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
       collected = g_slist_next (collected)) {
     GstMatroskaPad *collect_pad;
     GstFormat format = GST_FORMAT_TIME;
-    GstPad *thepad;
+    GstPad *thepad, *peerpad;
     gint64 trackduration;
 
     collect_pad = (GstMatroskaPad *) collected->data;
     thepad = collect_pad->collect.pad;
 
     /* Query the total length of the track. */
-    GST_DEBUG_OBJECT (thepad, "querying peer duration");
-    if (gst_pad_query_peer_duration (thepad, &format, &trackduration)) {
-      GST_DEBUG_OBJECT (thepad, "duration: %" GST_TIME_FORMAT,
+    peerpad = gst_pad_get_peer (thepad);
+    GST_DEBUG_OBJECT (thepad, "querying duration");
+    if (gst_pad_query_duration (peerpad, &format, &trackduration)) {
+      GST_DEBUG_OBJECT (thepad, "%duration: %" GST_TIME_FORMAT,
           GST_TIME_ARGS (trackduration));
-      if (trackduration != GST_CLOCK_TIME_NONE && trackduration > duration) {
-        duration = (GstClockTime) trackduration;
+      if ((gdouble) trackduration > duration) {
+        duration = (gdouble) trackduration;
       }
     }
+    gst_object_unref (peerpad);
   }
   gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
-      gst_guint64_to_gdouble (duration) /
-      gst_guint64_to_gdouble (mux->time_scale));
-
+      duration / gst_guint64_to_gdouble (mux->time_scale));
   gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_MUXINGAPP,
       "GStreamer plugin version " PACKAGE_VERSION);
   if (mux->writing_app && mux->writing_app[0]) {
@@ -1425,56 +1121,6 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   gst_ebml_write_flush_cache (ebml);
 }
 
-static void
-gst_matroska_mux_write_simple_tag (const GstTagList * list, const gchar * tag,
-    gpointer data)
-{
-  struct
-  {
-    gchar *matroska_tagname;
-    gchar *gstreamer_tagname;
-  }
-  tag_conv[] = {
-    {
-    GST_MATROSKA_TAG_ID_TITLE, GST_TAG_TITLE}, {
-    GST_MATROSKA_TAG_ID_AUTHOR, GST_TAG_ARTIST}, {
-    GST_MATROSKA_TAG_ID_ALBUM, GST_TAG_ALBUM}, {
-    GST_MATROSKA_TAG_ID_COMMENTS, GST_TAG_COMMENT}, {
-    GST_MATROSKA_TAG_ID_BITSPS, GST_TAG_BITRATE}, {
-    GST_MATROSKA_TAG_ID_DATE, GST_TAG_DATE}, {
-    GST_MATROSKA_TAG_ID_ISRC, GST_TAG_ISRC}, {
-    GST_MATROSKA_TAG_ID_COPYRIGHT, GST_TAG_COPYRIGHT}
-  };
-  GstEbmlWrite *ebml = (GstEbmlWrite *) data;
-  guint i;
-  guint64 simpletag_master;
-
-  for (i = 0; i < G_N_ELEMENTS (tag_conv); i++) {
-    const gchar *tagname_gst = tag_conv[i].gstreamer_tagname;
-    const gchar *tagname_mkv = tag_conv[i].matroska_tagname;
-
-    if (strcmp (tagname_gst, tag) == 0) {
-      GValue src = { 0, };
-      GValue dest = { 0, };
-
-      if (!gst_tag_list_copy_value (&src, list, tag))
-        break;
-      g_value_init (&dest, G_TYPE_STRING);
-      g_value_transform (&src, &dest);
-      g_value_unset (&src);
-
-      simpletag_master = gst_ebml_write_master_start (ebml,
-          GST_MATROSKA_ID_SIMPLETAG);
-      gst_ebml_write_ascii (ebml, GST_MATROSKA_ID_TAGNAME, tagname_mkv);
-      gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_TAGSTRING,
-          g_value_get_string (&dest));
-      gst_ebml_write_master_finish (ebml, simpletag_master);
-      g_value_unset (&dest);
-      break;
-    }
-  }
-}
-
 
 /**
  * gst_matroska_mux_finish:
@@ -1489,7 +1135,6 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   guint64 pos;
   guint64 duration = 0;
   GSList *collected;
-  GstTagList *tags;
 
   /* finish last cluster */
   if (mux->cluster) {
@@ -1547,21 +1192,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   }
   gst_ebml_write_flush_cache (ebml);
 
-  /* tags */
-  tags = gst_tag_list_merge (gst_tag_setter_get_tag_list (GST_TAG_SETTER (mux)),
-      mux->tags, GST_TAG_MERGE_APPEND);
-
-  if (tags != NULL) {
-    guint64 master_tags, master_tag;
-
-    mux->tags_pos = ebml->pos;
-    master_tags = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_TAGS);
-    master_tag = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_TAG);
-    gst_tag_list_foreach (tags, gst_matroska_mux_write_simple_tag, ebml);
-    gst_ebml_write_master_finish (ebml, master_tag);
-    gst_ebml_write_master_finish (ebml, master_tags);
-    gst_tag_list_free (tags);
-  }
+  /* FIXME: tags */
 
   /* update seekhead. We know that:
    * - a seekhead contains 4 entries.
@@ -1599,56 +1230,27 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
     gst_ebml_write_seek (ebml, my_pos);
   }
-  if (tags != NULL) {
-    gst_ebml_replace_uint (ebml, mux->seekhead_pos + 144,
-        mux->tags_pos - mux->segment_master);
-  } else {
-    /* void'ify */
-    guint64 my_pos = ebml->pos;
-
-    gst_ebml_write_seek (ebml, mux->seekhead_pos + 124);
-    gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
-    gst_ebml_write_seek (ebml, my_pos);
-  }
+#if 0
+  gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
+      mux->tags_pos - mux->segment_master);
+#endif
 
   /* update duration */
   /* first get the overall duration */
-  /* a released track may have left a duration in here */
-  duration = mux->duration;
   for (collected = mux->collect->data; collected;
       collected = g_slist_next (collected)) {
     GstMatroskaPad *collect_pad;
-    GstClockTime min_duration;  /* observed minimum duration */
 
     collect_pad = (GstMatroskaPad *) collected->data;
 
-    GST_DEBUG_OBJECT (mux, "Pad %" GST_PTR_FORMAT " start ts %" GST_TIME_FORMAT
-        " end ts %" GST_TIME_FORMAT, collect_pad,
-        GST_TIME_ARGS (collect_pad->start_ts),
-        GST_TIME_ARGS (collect_pad->end_ts));
-
-    if (GST_CLOCK_TIME_IS_VALID (collect_pad->start_ts) &&
-        GST_CLOCK_TIME_IS_VALID (collect_pad->end_ts)) {
-      min_duration =
-          GST_CLOCK_DIFF (collect_pad->start_ts, collect_pad->end_ts);
-      if (collect_pad->duration < min_duration)
-        collect_pad->duration = min_duration;
-      GST_DEBUG_OBJECT (collect_pad, "final track duration: %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (collect_pad->duration));
-    }
-
-    if (GST_CLOCK_TIME_IS_VALID (collect_pad->duration) &&
-        duration < collect_pad->duration)
+    if (collect_pad->duration > duration)
       duration = collect_pad->duration;
   }
   if (duration != 0) {
-    GST_DEBUG_OBJECT (mux, "final total duration: %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (duration));
     pos = mux->ebml_write->pos;
     gst_ebml_write_seek (ebml, mux->duration_pos);
     gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
-        gst_guint64_to_gdouble (duration) /
-        gst_guint64_to_gdouble (mux->time_scale));
+        gst_guint64_to_gdouble (duration / mux->time_scale));
     gst_ebml_write_seek (ebml, pos);
   }
 
@@ -1702,6 +1304,35 @@ gst_matroska_mux_best_pad (GstMatroskaMux * mux, gboolean * popped)
   return best;
 }
 
+static gboolean
+gst_matroska_mux_stream_is_vorbis_header (GstMatroskaMux * mux,
+    GstMatroskaPad * collect_pad)
+{
+  GstMatroskaTrackAudioContext *audio_ctx;
+
+  audio_ctx = (GstMatroskaTrackAudioContext *) collect_pad->track;
+
+  if (collect_pad->track->type != GST_MATROSKA_TRACK_TYPE_AUDIO)
+    return FALSE;
+
+  if (audio_ctx->first_frame != FALSE)
+    return FALSE;
+
+  if (collect_pad->track->codec_id == NULL ||
+      strcmp (collect_pad->track->codec_id, GST_MATROSKA_CODEC_ID_AUDIO_VORBIS))
+    return FALSE;
+
+  /* HACK: three frame headers are counted using pos */
+  if (++collect_pad->track->pos <= 3)
+    return TRUE;
+
+  /* 4th vorbis packet => skipped all headers */
+  collect_pad->track->pos = 0;
+  audio_ctx->first_frame = TRUE;
+  return FALSE;
+}
+
+
 /**
  * gst_matroska_mux_buffer_header:
  * @track: Track context.
@@ -1755,11 +1386,10 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
   buf = collect_pad->buffer;
   collect_pad->buffer = NULL;
 
-  /* vorbis/theora headers are retrieved from caps and put in CodecPrivate */
-  if (collect_pad->track->xiph_headers_to_skip > 0) {
-    GST_LOG_OBJECT (collect_pad->collect.pad, "dropping streamheader buffer");
+  /* vorbis header are retrieved from caps and placed in CodecPrivate */
+  if (gst_matroska_mux_stream_is_vorbis_header (mux, collect_pad)) {
+    GST_LOG_OBJECT (collect_pad->collect.pad, "dropping vorbis header buffer");
     gst_buffer_unref (buf);
-    --collect_pad->track->xiph_headers_to_skip;
     return GST_FLOW_OK;
   }
 
@@ -1956,29 +1586,7 @@ gst_matroska_mux_collected (GstCollectPads * pads, gpointer user_data)
       ret = GST_FLOW_UNEXPECTED;
       break;
     }
-    GST_DEBUG_OBJECT (best->collect.pad, "best pad - buffer ts %"
-        GST_TIME_FORMAT " dur %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (best->buffer)),
-        GST_TIME_ARGS (GST_BUFFER_DURATION (best->buffer)));
-
-    /* make note of first and last encountered timestamps, so we can calculate
-     * the actual duration later when we send an updated header on eos */
-    if (GST_BUFFER_TIMESTAMP_IS_VALID (best->buffer)) {
-      GstClockTime start_ts = GST_BUFFER_TIMESTAMP (best->buffer);
-      GstClockTime end_ts = start_ts;
-
-      if (GST_BUFFER_DURATION_IS_VALID (best->buffer))
-        end_ts += GST_BUFFER_DURATION (best->buffer);
-      else if (best->track->default_duration)
-        end_ts += best->track->default_duration;
-
-      if (end_ts > best->end_ts)
-        best->end_ts = end_ts;
-
-      if (G_UNLIKELY (best->start_ts == GST_CLOCK_TIME_NONE ||
-              start_ts < best->start_ts))
-        best->start_ts = start_ts;
-    }
+    GST_DEBUG_OBJECT (best->collect.pad, "best pad");
 
     /* write one buffer */
     ret = gst_matroska_mux_write_data (mux, best);
@@ -2011,9 +1619,6 @@ gst_matroska_mux_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_collect_pads_stop (mux->collect);
-      break;
     default:
       break;
   }
@@ -2024,6 +1629,7 @@ gst_matroska_mux_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_collect_pads_stop (mux->collect);
       gst_matroska_mux_reset (GST_ELEMENT (mux));
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
