@@ -1,4 +1,4 @@
-/* GStreamer
+/* Gnome-Streamer
  * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -31,10 +31,10 @@ G_BEGIN_DECLS
 #define GST_RTP_MP4G_PAY(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_RTP_MP4G_PAY,GstRtpMP4GPay))
 #define GST_RTP_MP4G_PAY_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_RTP_MP4G_PAY,GstRtpMP4GPayClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_RTP_MP4G_PAY,GstRtpMP4GPay))
 #define GST_IS_RTP_MP4G_PAY(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_RTP_MP4G_PAY))
-#define GST_IS_RTP_MP4G_PAY_CLASS(klass) \
+#define GST_IS_RTP_MP4G_PAY_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RTP_MP4G_PAY))
 
 typedef struct _GstRtpMP4GPay GstRtpMP4GPay;
@@ -45,14 +45,13 @@ struct _GstRtpMP4GPay
   GstBaseRTPPayload    payload;
 
   GstAdapter   *adapter;
-  GstClockTime  first_timestamp;
-  GstClockTime  first_duration;
+  GstClockTime  first_ts;
   GstClockTime  duration;
 
   gint          rate;
-  gchar        *params;
-  gchar        *profile;
-  const gchar  *streamtype;
+  gint		params;
+  gint          profile;
+  gint          streamtype;
   const gchar  *mode;
   GstBuffer    *config;
 };
