@@ -23,12 +23,11 @@
 #include "gstrtpspeexpay.h"
 
 /* elementfactory information */
-static GstElementDetails gst_rtp_speex_pay_details = {
-  "RTP packet parser",
-  "Codec/Payloader/Network",
-  "Payload-encodes Speex audio into a RTP packet",
-  "Edgard Lima <edgard.lima@indt.org.br>"
-};
+static const GstElementDetails gst_rtp_speex_pay_details =
+GST_ELEMENT_DETAILS ("RTP packet parser",
+    "Codec/Payloader/Network",
+    "Payload-encodes Speex audio into a RTP packet",
+    "Edgard Lima <edgard.lima@indt.org.br>");
 
 static GstStaticPadTemplate gst_rtp_speex_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -80,7 +79,7 @@ gst_rtp_speex_pay_class_init (GstRtpSPEEXPayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_BASE_RTP_PAYLOAD);
+  parent_class = g_type_class_peek_parent (klass);
 
   gstbasertppayload_class->set_caps = gst_rtp_speex_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_speex_pay_handle_buffer;
